@@ -14,6 +14,8 @@ import random
 import asyncio
 import time
 
+from pathlib import Path
+
 try:
     import ujson as json
 except ModuleNotFoundError:
@@ -24,13 +26,112 @@ from .config import Config
 
 # 加载全局配置
 global_config = nonebot.get_driver().config
-config = Config.parse_obj(global_config.dict())
+waifu_config = Config.parse_obj(global_config.dict())
+
+waifu_cd_bye = waifu_config.waifu_cd_bye
+
+waifu_save = waifu_config.waifu_save
+
+waifu_reset = waifu_config.waifu_reset
+
+HE = waifu_config.waifu_he
+BE = HE + waifu_config.waifu_be
+NTR = waifu_config.waifu_ntr
+
+yinpa_HE = waifu_config.yinpa_he
+yinpa_BE = yinpa_HE + waifu_config.yinpa_be
+yinpa_CP = waifu_config.yinpa_cp
+yinpa_CP = yinpa_HE if yinpa_CP == 0 else yinpa_CP
+
+
+waifu_file = Path() / "data" / "waifu"
+
+if not waifu_file.exists():
+    os.makedirs(waifu_file)
+
+record_waifu_file = waifu_file / "record_waifu"
+record_yinpa1_file = waifu_file / "record_yinpa1"
+record_yinpa2_file = waifu_file / "record_yinpa2"
+
+if waifu_save:
+    def save(file, data):
+        with open(file, "w", encoding="utf8") as f:
+            f.write(str(data))
+else:
+    def save(file, data):
+        pass
 
 scheduler = require("nonebot_plugin_apscheduler").scheduler
 
-# 娶群友
+if waifu_reset:
 
-record_waifu = {}
+    # 判断文件时效
+    timestr = time.strftime('%Y-%m-%d',time.localtime(time.time()))
+    timeArray = time.strptime(timestr,'%Y-%m-%d')
+    Zero_today = time.mktime(timeArray)
+
+    if record_waifu_file.exists() and os.path.getmtime(record_waifu_file) > Zero_today:
+        with open(record_waifu_file,'r') as f:
+            line = f.read()
+            record_waifu = eval(line)
+    else:
+        record_waifu = {}
+
+    if record_yinpa1_file.exists() and os.path.getmtime(record_yinpa1_file) > Zero_today:
+        with open(record_yinpa1_file,'r') as f:
+            line = f.read()
+            record_yinpa1 = eval(line)
+    else:
+        record_yinpa1 = {}
+
+    if record_yinpa2_file.exists() and os.path.getmtime(record_yinpa2_file) > Zero_today:
+        with open(record_yinpa2_file,'r') as f:
+            line = f.read()
+            record_yinpa2 = eval(line)
+    else:
+        record_yinpa2 = {}
+
+    # 重置记录
+
+    @scheduler.scheduled_job("cron",hour = 0)
+    def _():
+        global record_waifu,record_yinpa1,record_yinpa2
+        record_waifu = {}
+        record_yinpa1 = {}
+        record_yinpa2 = {}
+else:
+
+    if record_waifu_file.exists():
+        with open(record_waifu_file,'r') as f:
+            line = f.read()
+            record_waifu = eval(line)
+    else:
+        record_waifu = {}
+
+    if record_yinpa1_file.exists():
+        with open(record_yinpa1_file,'r') as f:
+            line = f.read()
+            record_yinpa1 = eval(line)
+    else:
+        record_yinpa1 = {}
+
+    if record_yinpa2_file.exists():
+        with open(record_yinpa2_file,'r') as f:
+            line = f.read()
+            record_yinpa2 = eval(line)
+    else:
+        record_yinpa2 = {}
+
+    # 重置记录
+    @scheduler.scheduled_job("cron",hour = 0)
+    def _():
+        global record_yinpa1,record_yinpa2
+        record_yinpa1 = {}
+        record_yinpa2 = {}
+
+
+
+# 娶群友
 
 waifu = on_command("娶群友", permission=GROUP, priority = 90, block = True)
 
@@ -52,58 +153,15 @@ happy_end= [
 async def _(bot:Bot, event: GroupMessageEvent):
     group_id = event.group_id
     user_id = event.user_id
-    global record_waifu
+    global record_waifu_file, record_waifu
     record_waifu.setdefault(group_id,{})
     at = get_message_at(event.json())
     if at and at[0] != user_id:
         at = at[0]
         if record_waifu[group_id].get(user_id,0) == 0:
-            X = random.randint(1,6)
             if record_waifu[group_id].get(at,0) in (0, at):
-                if X > 3:
-                    if X == 6:
-                        record_waifu[group_id].update(
-                            {
-                                user_id: at,
-                                at: user_id
-                                }
-                            )
-                        await waifu.send("恭喜你娶到了群友" + MessageSegment.at(at), at_sender=True)
-                        await asyncio.sleep(1)
-                    else:
-                        record_waifu[group_id][user_id] = user_id
-                else:
-                    pass
-            else:
-                try:
-                    member = await bot.get_group_member_info(group_id = group_id, user_id = record_waifu[group_id][at])
-                except:
-                    member = None
-                if member:
-                    if X == 6: # 彩蛋
-                        await waifu.send(
-                            "人家已经名花有主了~" + 
-                            MessageSegment.image(file = await user_img(record_waifu[group_id][at])) +
-                            "ta的CP：" + ( member['card'] or member['nickname'] ) + '\n'
-                            "但是...",
-                            at_sender=True
-                            )
-                        record_waifu[group_id].pop(record_waifu[group_id][at])
-                        record_waifu[group_id].update(
-                            {
-                                user_id: at,
-                                at: user_id
-                                }
-                            )
-                    else:
-                        await waifu.send(
-                            "人家已经名花有主啦！" + 
-                            MessageSegment.image(file = await user_img(record_waifu[group_id][at])) +
-                            "ta的CP：" + ( member['card'] or member['nickname'] ),
-                            at_sender=True
-                            )
-                else:
-                    record_waifu[group_id].pop(record_waifu[group_id][at])
+                X = random.randint(1,100)
+                if 0 < X <= HE:
                     record_waifu[group_id].update(
                         {
                             user_id: at,
@@ -111,6 +169,38 @@ async def _(bot:Bot, event: GroupMessageEvent):
                             }
                         )
                     await waifu.send("恭喜你娶到了群友" + MessageSegment.at(at), at_sender=True)
+                    await asyncio.sleep(1)
+                elif HE < X <= BE:
+                    record_waifu[group_id][user_id] = user_id
+                else:
+                    pass
+            else:
+                try:
+                    member = await bot.get_group_member_info(group_id = group_id, user_id = record_waifu[group_id][at])
+                except:
+                    member = None
+                if random.randint(1,100) <= NTR: # 彩蛋
+                    record_waifu[group_id].pop(record_waifu[group_id][at])
+                    record_waifu[group_id].update(
+                        {
+                            user_id: at,
+                            at: user_id
+                            }
+                        )
+                    await waifu.send(
+                        "人家已经名花有主了~" + 
+                        MessageSegment.image(file = await user_img(record_waifu[group_id][at])) +
+                        "ta的CP：" + ( member['card'] or member['nickname'] ) + '\n'
+                        "但是...",
+                        at_sender=True
+                        )
+                else:
+                    await waifu.send(
+                        "人家已经名花有主啦！" + 
+                        MessageSegment.image(file = await user_img(record_waifu[group_id][at])) +
+                        "ta的CP：" + ( member['card'] or member['nickname'] ),
+                        at_sender=True
+                        )
                 await asyncio.sleep(1)
         elif record_waifu[group_id][user_id] == at:
             await waifu.finish(
@@ -135,6 +225,7 @@ async def _(bot:Bot, event: GroupMessageEvent):
                     )
             else:
                 record_waifu[group_id][user_id] = user_id
+
     if record_waifu[group_id].get(user_id,0) == 0:
         member_list = await bot.get_group_member_list(group_id = event.group_id)
         i = 0
@@ -155,24 +246,21 @@ async def _(bot:Bot, event: GroupMessageEvent):
                     )
                 nickname = member['card'] or member['nickname']
                 if record_waifu[group_id][user_id] == user_id:
-                    await waifu.finish(random.choice(no_waifu), at_sender=True)
+                    msg = random.choice(no_waifu)
                 else:
-                    await waifu.finish(
-                        (
-                            "さん的群友結婚对象是、\n",
-                            MessageSegment.image(file = await user_img(record_waifu[group_id][user_id])),
-                            f"『{nickname}』です！"
-                            ),
-                            at_sender=True
-                            )
+                    msg = (
+                        "的群友結婚对象是、\n",
+                        MessageSegment.image(file = await user_img(record_waifu[group_id][user_id])),
+                        f"『{nickname}』！"
+                        )
             else:
                 record_waifu[group_id][user_id] = 1
-                await waifu.finish("群友已经被娶光了、\n" + random.choice(no_waifu), at_sender=True)
+                msg = "群友已经被娶光了、\n" + random.choice(no_waifu)
     else:
         if record_waifu[group_id][user_id] == event.user_id:
-            await waifu.finish(random.choice(no_waifu), at_sender=True)
+            msg = random.choice(no_waifu)
         elif record_waifu[group_id][user_id] == 1:
-            await waifu.finish("群友已经被娶光了、\n" + random.choice(no_waifu), at_sender=True)
+            msg = "群友已经被娶光了、\n" + random.choice(no_waifu)
         else:
             try:
                 member = await bot.get_group_member_info(group_id = group_id, user_id = record_waifu[group_id][user_id])
@@ -180,23 +268,23 @@ async def _(bot:Bot, event: GroupMessageEvent):
                 member = None
             if member:
                 nickname = member['card'] or member['nickname']
-                await waifu.finish(
-                    (
-                        "さん的群友結婚对象是、\n",
-                        MessageSegment.image(file = await user_img(record_waifu[group_id][user_id])),
-                        f"『{nickname}』です！"
-                        ),
-                        at_sender=True
-                        )
+                msg = (
+                    "的群友結婚对象是、\n",
+                    MessageSegment.image(file = await user_img(record_waifu[group_id][user_id])),
+                    f"『{nickname}』！"
+                    )
             else:
-                await waifu.finish(random.choice(no_waifu), at_sender=True)
+                msg = random.choice(no_waifu)
+
+    save(record_waifu_file,record_waifu)
+    await waifu.finish(msg, at_sender=True)
 
 # 分手
 
 async def FACTOR(bot: Bot, event: GroupMessageEvent) -> bool:
     global record_waifu
     record_waifu.setdefault(event.group_id,{})
-    return record_waifu[event.group_id].get(event.user_id,0) not in (0, 1, event.user_id) and config.waifu_cd_bye != -1
+    return record_waifu[event.group_id].get(event.user_id,0) not in (0, 1, event.user_id) and waifu_cd_bye != -1
 
 global cd_bye
 cd_bye = {}
@@ -205,17 +293,18 @@ bye = on_command("离婚", aliases = {"分手"}, permission = FACTOR, priority =
 
 @bye.handle()
 async def _(bot:Bot, event: GroupMessageEvent):
-    global record_waifu, cd_bye
+    global record_waifu_file, record_waifu, cd_bye
     cd_bye.setdefault(event.group_id,{})
     flag = cd_bye[event.group_id].setdefault(event.user_id,[0,0])
     Now = time.time()
     cd = flag[0] - Now
     if cd <= 0:
-        cd_bye[event.group_id][event.user_id][0] = Now + config.waifu_cd_bye
+        cd_bye[event.group_id][event.user_id][0] = Now + waifu_cd_bye
         A = event.user_id
         B = int(record_waifu[event.group_id][event.user_id])
         del record_waifu[event.group_id][A]
         del record_waifu[event.group_id][B]
+        save(record_waifu_file,record_waifu)
         if random.randint(1,2) == 1:
             await bye.finish(random.choice(("嗯。","...","好。")))
         else:
@@ -301,30 +390,72 @@ async def _(bot:Bot, event: GroupMessageEvent):
 
 # 透群友
 
-record_yinpa = {}
-
 yinpa = on_command("透群友", permission=GROUP, priority = 90, block = True)
 
 @yinpa.handle()
 async def _(bot:Bot, event: GroupMessageEvent):
     group_id = event.group_id
     user_id = event.user_id
-    member_list = await bot.get_group_member_list(group_id = event.group_id)
-    member_list.sort(key = lambda x:x["last_sent_time"] ,reverse = True)
-    member = random.choice(member_list[:80])
+    global record_yinpa1,record_yinpa2
+    at = get_message_at(event.json())
+    msg = ""
+    if at:
+        if at[0] == user_id:
+            pass
+        elif at[0] == record_waifu[group_id].get(user_id,0):
+            X = random.randint(1,100)
+            if 0 < X <= yinpa_CP:
+                member = await bot.get_group_member_info(group_id = group_id, user_id = at[0])
+                nickname = member['card'] or member['nickname']
+                record_yinpa1.setdefault(user_id,0)
+                record_yinpa1[user_id] += 1
+                record_yinpa2.setdefault(member['user_id'],0)
+                record_yinpa2[member['user_id']] += 1
+                msg = (
+                    f"恭喜你涩到了你的老婆\n",
+                    MessageSegment.image(file = await user_img(member["user_id"])),
+                    f"『{nickname}』！"
+                    )
+            else:
+                msg = "你的老婆拒绝和你涩涩！"
+        else:
+            X = random.randint(1,100)
+            if 0 < X <= yinpa_HE:
+                member = await bot.get_group_member_info(group_id = group_id, user_id = at[0])
+                nickname = member['card'] or member['nickname']
+                record_yinpa1.setdefault(user_id,0)
+                record_yinpa1[user_id] += 1
+                record_yinpa2.setdefault(member['user_id'],0)
+                record_yinpa2[member['user_id']] += 1
+                msg = (
+                    f"恭喜你涩到了群友\n",
+                    MessageSegment.image(file = await user_img(member["user_id"])),
+                    f"『{nickname}』！"
+                    )
+            elif yinpa_HE < X < yinpa_BE:
+                msg = "不可以涩涩！"
+            else:
+                pass
+    if not msg:
+        member_list = await bot.get_group_member_list(group_id = event.group_id)
+        member_list.sort(key = lambda x:x["last_sent_time"] ,reverse = True)
+        member = random.choice(member_list[:80])
+        if member["user_id"] == event.user_id:
+            msg = "不可以涩涩！"
+        else:
+            nickname = member['card'] or member['nickname']
+            record_yinpa1.setdefault(user_id,0)
+            record_yinpa1[user_id] += 1
+            record_yinpa2.setdefault(member['user_id'],0)
+            record_yinpa2[member['user_id']] += 1
+            msg = (
+                "的涩涩对象是、\n",
+                MessageSegment.image(file = await user_img(member["user_id"])),
+                f"『{nickname}』！"
+                )
 
-    if member["user_id"] == event.user_id:
-        msg = "不可以涩涩！"
-    else:
-        nickname = member['card'] or member['nickname']
-        global record_yinpa
-        record_yinpa.setdefault(member['user_id'],0)
-        record_yinpa[member['user_id']] += 1
-        msg = (
-            "さんが群友で涩涩するであろうヒロインは、\n",
-            MessageSegment.image(file = await user_img(member["user_id"])),
-            f"『{nickname}』です！"
-            )
+    save(record_yinpa1_file,record_yinpa1)
+    save(record_yinpa2_file,record_yinpa2)
     await yinpa.finish(msg, at_sender=True)
 
 # 查看涩涩记录
@@ -333,47 +464,83 @@ yinpa_list = on_command("涩涩记录",aliases = {"色色记录"}, permission=GR
 
 @yinpa_list.handle()
 async def _(bot:Bot, event: GroupMessageEvent):
-    member_list = await bot.get_group_member_list(group_id = event.group_id)
-    member_list.sort(key = lambda x:x["last_sent_time"] ,reverse = True)
-    record = []
-    for member in member_list:
-        nickname = member['card'] or member['nickname']
-        global record_yinpa
-        times = record_yinpa.get(member['user_id'],0)
-        if times:
-            record.append([nickname,times])
-    else:
-        record.sort(key = lambda x:x[1],reverse = True)
+    global record_yinpa1,record_yinpa2
 
     msg_list =[]
+
+    # 输出卡池
+    member_list = await bot.get_group_member_list(group_id = event.group_id)
+    member_list.sort(key = lambda x:x["last_sent_time"] ,reverse = True)
+
     msg ="卡池：\n——————————————\n"
     for member in member_list[:80]:
         nickname = member['card'] or member['nickname']
         msg += f"{nickname}\n"
-    else:
-        output = text_to_png(msg[:-1])
-        msg_list.append(
-            {
-                "type": "node",
-                "data": {
-                    "name": "卡池",
-                    "uin": event.self_id,
-                    "content": MessageSegment.image(output)
-                    }
-                }
-            )
 
-    msg =""
+    output = text_to_png(msg[:-1])
+    msg_list.append(
+        {
+            "type": "node",
+            "data": {
+                "name": "卡池",
+                "uin": event.self_id,
+                "content": MessageSegment.image(output)
+                }
+            }
+        )
+
+    # 输出透群友记录
+
+    record = []
+    for member in member_list:
+        nickname = member['card'] or member['nickname']
+        times = record_yinpa1.get(member['user_id'],0)
+        if times:
+            record.append([nickname,times])
+
+    record.sort(key = lambda x:x[1],reverse = True)
+
+    msg = "涩涩记录①：\n——————————————\n"
     for info in record:
-        msg += f"【{info[0]}】\n今日被透 {info[1]} 次\n"
+        msg += f"[align=left]{info[0]}[/align][align=right]今日透群友 {info[1]} 次[/align]\n"
     else:
         if msg:
-            output = text_to_png("涩涩记录：\n——————————————\n" + msg[:-1])
+            output = bbcode_to_png(msg[:-1])
             msg_list.append(
                 {
                     "type": "node",
                     "data": {
-                        "name": "记录",
+                        "name": "记录①",
+                        "uin": event.self_id,
+                        "content": MessageSegment.image(output)
+                        }
+                    }
+                )
+        else:
+            pass
+
+    # 输出被透记录
+
+    record = []
+    for member in member_list:
+        nickname = member['card'] or member['nickname']
+        times = record_yinpa2.get(member['user_id'],0)
+        if times:
+            record.append([nickname,times])
+
+    record.sort(key = lambda x:x[1],reverse = True)
+
+    msg = "涩涩记录②：\n——————————————\n"
+    for info in record:
+        msg += f"[align=left]{info[0]}[/align][align=right]今日被透 {info[1]} 次[/align]\n"
+    else:
+        if msg:
+            output = bbcode_to_png(msg[:-1])
+            msg_list.append(
+                {
+                    "type": "node",
+                    "data": {
+                        "name": "记录②",
                         "uin": event.self_id,
                         "content": MessageSegment.image(output)
                         }
@@ -382,12 +549,4 @@ async def _(bot:Bot, event: GroupMessageEvent):
         else:
             pass
     await bot.send_group_forward_msg(group_id = event.group_id, messages = msg_list)
-    await yinpa_list.finish()
-
-# 重置娶群友记录
-
-@scheduler.scheduled_job("cron",hour = 0)
-def _():
-    global record_waifu,record_yinpa
-    record_waifu = {}
-    record_yinpa = {}
\ No newline at end of file
+    await yinpa_list.finish()
\ No newline at end of file
